// ------------------------------------------------------------------------------------
// <copyright file="CiudadesBL.cs" company="Universidad Distrital">
// Copyright (c) Universidad Distrital. All rights reserved.
// </copyright>
// <author>Camilo Prieto</author>
// ------------------------------------------------------------------------------------
namespace Biblioteca.Negocio.Clases.BL
{
	using System;
	using System.Collections.Generic;
	using System.Threading.Tasks;
	using Biblioteca.Datos.Clases.DAL.Repositorio;
	using Biblioteca.IC.Acciones.Repositorio;
	using Biblioteca.IC.DTO.Repositorio;
	using Biblioteca.IC.Recursos;
	using TransaccionalNet.EF.Clases;
	using UtilitariosNet.Clases.Comunes.Entidades;
	using Utilitarios.TransversalesNet.Clases.Comunes.Enumeraciones;

	/// <summary>
	/// Clase con las acciones de negocio de la entidad Ciudades
	/// </summary>
	public class CiudadesBL : AccesoComunBL, ICiudadesNegocioAccion
	{

		#region ATRIBUTOS
		/// <summary>
		/// Objeto para repositorio Ciudades
		/// <summary>
		/// Objeto para acciones Ciudades
		/// </summary>
		private Lazy<ICiudadesRepositorioAccion> ciudadesRepositorioAccion;

		/// <summary>
		/// Objeto para entidad respuesta
		/// </summary>
		Respuesta<ICiudadesDTO> respuesta;

		#endregion ATRIBUTOS
		#region CONSTRUCTORES
		/// <summary>
		/// Inicializa una nueva instancia de la clase CiudadesBL
		/// </summary>
		/// <param name="argCiudadesRepositorioAccion">Acciones entidad Ciudades</param>
		public CiudadesBL(Lazy<ICiudadesRepositorioAccion> argCiudadesRepositorioAccion = null)
		{
			this.respuesta = new Respuesta<ICiudadesDTO>();
			this.ciudadesRepositorioAccion = argCiudadesRepositorioAccion ?? new Lazy<ICiudadesRepositorioAccion>(() => new CiudadesDAL());
		}

		#endregion CONSTRUCTORES
		#region METODOS PUBLICOS
		/// <summary>
		/// Metodo consultar lista ciudades
		/// </summary>
		/// <returns>Respuesta tipo Ciudades </returns>
		public async Task<Respuesta<ICiudadesDTO>> ConsultarListaCiudadesAsync()
		{

			return await this.EjecutarTransaccionBDAsync<Respuesta<ICiudadesDTO>, CiudadesBL>(
			System.Transactions.IsolationLevel.ReadUncommitted,
			async () =>
			{
				 respuesta= await this.ciudadesRepositorioAccion.Value.ConsultarListaCiudadesAsync();
				 respuesta.Resultado = true;
				 respuesta.TipoNotificacion = TipoNotificacion.Exitoso;
				 return respuesta;

			});

		}
		/// <summary>
		/// Metodo consultar por llave ciudades
		/// </summary>
		/// <param name="ciudades">Entidad a consultar</param>
		/// <returns>Respuesta tipo Ciudades </returns>
		public async Task<Respuesta<ICiudadesDTO>> ConsultarCiudadesLlaveAsync(ICiudadesDTO ciudades)
		{
			return await this.EjecutarTransaccionBDAsync<Respuesta<ICiudadesDTO>, CiudadesBL>(
			System.Transactions.IsolationLevel.ReadUncommitted,
			async () =>
			{
				 respuesta= await this.ciudadesRepositorioAccion.Value.ConsultarCiudadesLlaveAsync(ciudades);
				 respuesta.Resultado = true;
				 respuesta.TipoNotificacion = TipoNotificacion.Exitoso;
				 return respuesta;

			});
		}

		/// <summary>
		/// Metodo editar ciudades
		/// </summary>
		/// <param name="ciudades">Entidad a editar</param>
		/// <returns>Respuesta tipo Ciudades </returns>
		public async Task<Respuesta<ICiudadesDTO>> EditarCiudadesAsync(ICiudadesDTO ciudades)
		{
			return await this.EjecutarTransaccionBDAsync<Respuesta<ICiudadesDTO>, CiudadesBL>(
			System.Transactions.IsolationLevel.ReadUncommitted,
			async () =>
			{
				 respuesta= await this.ciudadesRepositorioAccion.Value.EditarCiudadesAsync(ciudades);
				 respuesta.Resultado = true;
				 respuesta.TipoNotificacion = TipoNotificacion.Exitoso;
				 respuesta.Mensajes.Add(rcsMensajesComunes.MensajeCreacionEdicionExitosa);
				 return respuesta;

			});
		}


		/// <summary>
		/// Metodo eliminar ciudades
		/// </summary>
		/// <param name="ciudades">Entidad a eliminar</param>
		/// <returns>Respuesta tipo Ciudades </returns>
		public async Task<Respuesta<ICiudadesDTO>> EliminarCiudadesAsync(ICiudadesDTO ciudades)
		{
			return await this.EjecutarTransaccionBDAsync<Respuesta<ICiudadesDTO>, CiudadesBL>(
			System.Transactions.IsolationLevel.ReadUncommitted,
			async () =>
			{
				 respuesta= await this.ciudadesRepositorioAccion.Value.EliminarCiudadesAsync(ciudades);
				 respuesta.Resultado = true;
				 respuesta.TipoNotificacion = TipoNotificacion.Exitoso;
				 respuesta.Mensajes.Add(rcsMensajesComunes.MensajeEntidadEliminadaConExito);
				 return respuesta;

			});
		}

		/// <summary>
		/// Metodo guardar ciudades
		/// </summary>
		/// <param name="ciudades">Entidad a guardar</param>
		/// <returns>Respuesta tipo Ciudades </returns>
		public async Task<Respuesta<ICiudadesDTO>> GuardarCiudadesAsync(ICiudadesDTO ciudades)
		{
			return await this.EjecutarTransaccionBDAsync<Respuesta<ICiudadesDTO>, CiudadesBL>(
			System.Transactions.IsolationLevel.ReadUncommitted,
			async () =>
			{
				 respuesta= await this.ciudadesRepositorioAccion.Value.GuardarCiudadesAsync(ciudades);
				 respuesta.Resultado = true;
				 respuesta.TipoNotificacion = TipoNotificacion.Exitoso;
				 respuesta.Mensajes.Add(rcsMensajesComunes.MensajeCreacionEdicionExitosa);
				 return respuesta;

			});
		}

		/// <summary>
		/// Metodo Guardar lista ciudades
		/// </summary>
		/// <param name="listaCiudades">Lista de entidades a guardar</param>
		/// <returns>Respuesta tipo Ciudades </returns>
		public async Task<Respuesta<ICiudadesDTO>> GuardarListaCiudadesAsync(List<ICiudadesDTO> listaCiudades)
		{
			return await this.EjecutarTransaccionBDAsync<Respuesta<ICiudadesDTO>, CiudadesBL>(
			System.Transactions.IsolationLevel.ReadUncommitted,
			async () =>
			{
				 respuesta= await this.ciudadesRepositorioAccion.Value.GuardarListaCiudadesAsync(listaCiudades);
				 respuesta.Resultado = true;
				 respuesta.TipoNotificacion = TipoNotificacion.Exitoso;
				 respuesta.Mensajes.Add(rcsMensajesComunes.MensajeCreacionEdicionExitosa);
				 return respuesta;

			});
		}
		#endregion METODOS PUBLICOS
		#region METODOS PRIVADOS
		#endregion METODOS PRIVADOS
	}
}

