// ------------------------------------------------------------------------------------
// <copyright file="CiudadesController.cs" company="Universidad Distrital">
// Copyright (c) Universidad Distrital. All rights reserved.
// </copyright>
// <author>Camilo Prieto</author>
// ------------------------------------------------------------------------------------
namespace Biblioteca.API.Controllers
{
	using System;
	using System.Collections.Generic;
	using System.Threading.Tasks;
	using Utilitarios.APINet.Clases.Ejecucion;
	using UtilitariosNet.Clases.Comunes.Entidades;
	using UtilitariosNet.Clases.Mapeador.Extensiones;
	using Biblioteca.API.Models.Repositorio;
	using Biblioteca.IC.Acciones.Repositorio;
	using Biblioteca.Negocio.Clases.BL;
	using Microsoft.AspNetCore.Mvc;
	
	/// <summary>
	/// Clase con las capaciades Rest de la entidad Ciudades
	/// </summary>
	[Route("api/Ciudades")]
	public class CiudadesController : AccesoComunAPI
	{


		/// <summary>
		/// Objeto para negocio Ciudades
		/// </summary>
		private Lazy<ICiudadesNegocioAccion> negocioCiudades;

		
		/// <summary>
		/// Inicializa una nueva instancia de la clase CiudadesController
		/// </summary>
		public CiudadesController()
		{
			this.negocioCiudades = new Lazy<ICiudadesNegocioAccion>(
										() => new CiudadesBL());
		}
		
		/// <summary>
		/// Metodo consultar lista ciudades
		/// </summary>
		/// <returns>Respuesta tipo Ciudades </returns>
		[HttpGet]
		[Route("ConsultarListaCiudades")]
		public async Task<Respuesta<Ciudades>> ConsultarListaCiudades()
		{
			return await this.EjecutarTransaccionAPI<Task<Respuesta<Ciudades>>, CiudadesController>(async () =>
			{
				return Mapeador.MapearObjetoPorJson<Respuesta<Ciudades>>(await negocioCiudades.Value.ConsultarListaCiudadesAsync());
			});
		}
		
		/// <summary>
		/// Metodo consultar por llave ciudades
		/// </summary>
		/// <param name="ciudades">Entidad a consultar</param>
		/// <returns>Respuesta tipo Ciudades </returns>
		[HttpPost]
		[Route("ConsultarCiudadesLlave")]
		public async Task<Respuesta<Ciudades>> ConsultarCiudadesLlave(Ciudades ciudades)
		{
			return await this.EjecutarTransaccionAPI<Task<Respuesta<Ciudades>>, CiudadesController>(async () =>
			{
				return Mapeador.MapearObjetoPorJson<Respuesta<Ciudades>>(await negocioCiudades.Value.ConsultarCiudadesLlaveAsync(ciudades));
			});
		}
		
		/// <summary>
		/// Metodo guardar ciudades
		/// </summary>
		/// <param name="ciudades">Entidad a guardar</param>
		/// <returns>Respuesta tipo Ciudades </returns>
		[HttpPost]
		[Route("GuardarCiudades")]
		public async Task<Respuesta<Ciudades>> GuardarCiudades(Ciudades ciudades)
		{
			return await this.EjecutarTransaccionAPI<Task<Respuesta<Ciudades>>, CiudadesController>(async () =>
			{
				return Mapeador.MapearObjetoPorJson<Respuesta<Ciudades>>(await negocioCiudades.Value.GuardarCiudadesAsync(ciudades));
			});
		}
		
		/// <summary>
		/// Metodo editar ciudades
		/// </summary>
		/// <param name="ciudades">Entidad a editar</param>
		/// <returns>Respuesta tipo Ciudades </returns>
		[HttpPut]
		[Route("EditarCiudades")]
		public async Task<Respuesta<Ciudades>> EditarCiudades(Ciudades ciudades)
		{
			return await this.EjecutarTransaccionAPI<Task<Respuesta<Ciudades>>, CiudadesController>(async () =>
			{
				return Mapeador.MapearObjetoPorJson<Respuesta<Ciudades>>(await negocioCiudades.Value.EditarCiudadesAsync(ciudades));
			});
		}
		
		/// <summary>
		/// Metodo eliminar ciudades
		/// </summary>
		/// <param name="ciudades">Entidad a eliminar</param>
		/// <returns>Respuesta tipo Ciudades </returns>
		[HttpDelete]
		[Route("EliminarCiudades")]
		public async Task<Respuesta<Ciudades>> EliminarCiudades(Ciudades ciudades)
		{
			return await this.EjecutarTransaccionAPI<Task<Respuesta<Ciudades>>, CiudadesController>(async () =>
			{
				return Mapeador.MapearObjetoPorJson<Respuesta<Ciudades>>(await negocioCiudades.Value.EliminarCiudadesAsync(ciudades));
			});
		}
	}
}

